@@ -3,6 +3,8 @@
 Regression test for:
 https://bitbucket.org/logilab/pylint/issue/93/pylint-crashes-on-namedtuple-attribute
 """
+from __future__ import absolute_import, print_function
+
 __revision__ = None
 
 from collections import namedtuple
@@ -12,10 +14,10 @@ Fantastic = namedtuple('Fantastic', ['foo'])
 
 def test():
     """Test member access in named tuples."""
-    print Thing.x  # [no-member]
+    print(Thing.x)  # [no-member]
     fan = Fantastic(1)
-    print fan.foo
+    print(fan.foo)
     # Should not raise protected-access.
-    fan2 = fan._replace(foo=2)  # [protected-access]
+    fan2 = fan._replace(foo=2)
     # This is a bug.
-    print fan2.foo  # [no-member]
+    print(fan2.foo)  # [no-member]
