@@ -1,4 +1,5 @@
 """test min methods"""
+from __future__ import print_function
 __revision__ = None
 
 class Aaaa(object):
@@ -7,7 +8,7 @@ class Aaaa(object):
         pass
     def meth1(self):
         """hehehe"""
-        print self
+        print(self)
     def _dontcount(self):
         """not public"""
-        print self
+        print(self)
