@@ -1,5 +1,5 @@
-##########################################################################################
-""" that one is too long tooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo"""
+#####################################################################################################
+""" that one is too long tooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo loooooong"""
 
 __revision__ = ''
 
@@ -14,7 +14,7 @@ THIS_IS_A_VERY_LONG_VARIABLE_NAME = 'Существительное Частиц
 # line longer than 80 characters is a trailing pylint disable.
 var = 'This line has a disable pragma and whitespace trailing beyond 80 chars. '  # pylint:disable=invalid-name
 
-badname = 'This line is already longer than 80 characters even without the pragma.'  # pylint:disable=invalid-name
+badname = 'This line is already longer than 100 characters even without the pragma. Trust me. Please.'  # pylint:disable=invalid-name
 
 # http://example.com/this/is/a/very/long/url?but=splitting&urls=is&a=pain&so=they&can=be&long
 
@@ -22,6 +22,6 @@ badname = 'This line is already longer than 80 characters even without the pragm
 def function():
     """This is a docstring.
 
-    That contains a very, very long line that exceeds the 80 character limit by a good margin.
+    That contains a very, very long line that exceeds the 100 characters limit by a good margin. So good?
     """
     pass
