@@ -1,21 +1,21 @@
-# pylint: disable=R0903,W0212,W0403,W0406
+# pylint: disable=R0903,W0212,W0403,W0406,no-absolute-import
 """test for call to __init__ from a non ancestor class
 """
-
+from __future__ import print_function
 __revision__ = '$Id: func_w0233.py,v 1.2 2004-09-29 08:35:13 syt Exp $'
 
 class AAAA(object):
     """ancestor 1"""
 
     def __init__(self):
-        print 'init', self
+        print('init', self)
         BBBBMixin.__init__(self)
 
 class BBBBMixin(object):
     """ancestor 2"""
 
     def __init__(self):
-        print 'init', self
+        print('init', self)
 
 import nonexistant
 import func_w0233
@@ -34,3 +34,17 @@ class DDDD(AAAA):
             AAAA.__init__(self)
         else:
             AAAA.__init__(self)
+
+class Super(dict):
+    """ test late binding super() call """
+    def __init__(self):
+        base = super()
+        base.__init__()
+
+class Super2(dict):
+    """ Using the same idiom as Super, but without calling
+    the __init__ method.
+    """
+    def __init__(self):
+        base = super()
+        base.__woohoo__()
