@@ -1,4 +1,5 @@
 """Unittest for the type checker."""
+import unittest
 
 from astroid import test_utils
 from pylint.checkers import typecheck
@@ -38,5 +39,4 @@ class TypeCheckerTest(CheckerTestCase):
 
 
 if __name__ == '__main__':
-    from logilab.common.testlib import unittest_main
-    unittest_main()
+    unittest.main()
